CREATE DATABASE VirtualEarth		--Database dos exercicios
GO
USE VirtualEarth
GO

--Tabela de Veiculos
CREATE TABLE VEICULOS(ID INT PRIMARY KEY, VEICULO VARCHAR(50), DESCRICAO VARCHAR(100))
--Insere os 4 veiculos utilizados na simulao
INSERT VEICULOS VALUES(1, 'AMBULANCIA 1', 'AMBULANCIA SIMPLES')
INSERT VEICULOS VALUES(2, 'AMBULANCIA 2', 'AMBULANCIA UTI')
INSERT VEICULOS VALUES(3, 'AMBULANCIA 3', 'AMBULANCIA UTI')
INSERT VEICULOS VALUES(4, 'RESGATE'     , 'VEICULO DE RESGATE AUTONOMO')
GO

--Tabela de pontos geogrficos para ser utilizada integrada com a aplicao Virtual Earth
CREATE TABLE POSICAO(ID_VEICULO INT, ATUALIZACAO DATETIME DEFAULT GETDATE(), LOCALIZACAO geography)
GO
--View que traz a ultima data de cada veiculo
CREATE VIEW VW_ULTIMAATUALIZACAO(ID, VEICULO, DESCRICAO, ULTIMAPOSICAO) AS
	SELECT VEICULOS.ID, VEICULOS.VEICULO, VEICULOS.DESCRICAO, MAX(POSICAO.ATUALIZACAO)
	  FROM POSICAO INNER JOIN VEICULOS
			ON POSICAO.ID_VEICULO = VEICULOS.ID
	 GROUP BY VEICULOS.ID, VEICULOS.VEICULO, VEICULOS.DESCRICAO 
GO
--SP que grava a posicao enviada pelos veiculos
CREATE PROC PROC_GRAVAPOSICAO(@ID_VEICULO INT, @LONGITUDE FLOAT, @LATITUDE FLOAT) AS
	DECLARE @POSICAO VARCHAR(30)
	SET @POSICAO = 'POINT(' + CONVERT(VARCHAR(11), @LONGITUDE) + ' ' + CONVERT(VARCHAR(11), @LATITUDE) + ')'
	INSERT POSICAO(ID_VEICULO, LOCALIZACAO)
	 VALUES(@ID_VEICULO, geography::STPointFromText(@POSICAO, 4326))
GO
--SP que retorna os dados do veiculo com a ultima localizao
CREATE PROC PROC_LEPOSICAO AS
	SELECT VUA.*, POSICAO.LOCALIZACAO,
		   GEOGRAPHY::STPointFromText('POINT(-46.52427792549135 -23.442606611886766)', 4326) AS Matriz
	  FROM VW_ULTIMAATUALIZACAO AS VUA INNER JOIN POSICAO
			ON VUA.ID = POSICAO.ID_VEICULO AND VUA.ULTIMAPOSICAO = POSICAO.ATUALIZACAO
GO
--Testa as SPs
EXECUTE PROC_GRAVAPOSICAO 1, -46.482210159301765, -23.432654693575674
EXECUTE PROC_GRAVAPOSICAO 2, -46.658957004547126, -23.631864910301946
EXECUTE PROC_GRAVAPOSICAO 3, -46.719939708709724, -23.60039813406027
EXECUTE PROC_GRAVAPOSICAO 4, -46.566238403320305, -23.526326569485757
GO
EXECUTE PROC_LEPOSICAO
GO
SELECT * FROM POSICAO
GO

DECLARE @VEICULO1 GEOGRAPHY
SELECT @VEICULO1 = LOCALIZACAO FROM POSICAO WHERE ID_VEICULO = 1
DECLARE @VEICULO2 GEOGRAPHY
SELECT @VEICULO2 = LOCALIZACAO FROM POSICAO WHERE ID_VEICULO = 3
SELECT @VEICULO2.STDistance(@VEICULO1)

GO
TRUNCATE TABLE POSICAO
--D permisso pblica para execuo das SP pelo WebService
GRANT EXECUTE ON PROC_LEPOSICAO    TO PUBLIC
GRANT EXECUTE ON PROC_GRAVAPOSICAO TO PUBLIC
GO
